import 'dart:async';
import 'package:flutter/services.dart';

class ZxxIm {
  factory ZxxIm() {
    if (_instance == null) {
      final MethodChannel methodChannel = const MethodChannel('im_method');
      final EventChannel eventChannel = const EventChannel('im_event');
      _instance = new ZxxIm.private(methodChannel, eventChannel);
    }
    return _instance;
  }

  ZxxIm.private(this._methodChannel, this._eventChannel);

  final MethodChannel _methodChannel;

  final EventChannel _eventChannel;

  Future<String> get platformVersion async {
    final String version =
        await _methodChannel.invokeMethod('getPlatformVersion');
    return version;
  }

  static ZxxIm _instance;

  Stream<dynamic> _listener;

  Stream<dynamic> get onMessage {
    if (_listener == null) {
      _listener = _eventChannel
          .receiveBroadcastStream()
          .map((dynamic event) => _parseBatteryState(event));
    }
    return _listener;
  }

  ///im初始化
  ///Im initialization
  Future<dynamic> init(int appId) async {
    return await _methodChannel.invokeMethod("init", <String, dynamic>{
      'appid': appId,
    });
  }

  ///im登录
  ///Im login
  Future<dynamic> imLogin(String identifier, String sig) async {
    return await _methodChannel.invokeMethod("im_login", <String, dynamic>{
      'identifier': identifier,
      'userSig': sig,
    });
  }

  ///im登出
  ///Im log out
  Future<dynamic> imLogout() async {
    return await _methodChannel.invokeMethod("im_logout");
  }

  ///获取会话列表
  ///Get a list of conversations
  Future<dynamic> getConversations() async {
    return await _methodChannel.invokeMethod('getConversations');
  }

  ///删除会话
  ///Delete session
  Future<dynamic> delConversation(String identifier) async {
    return await _methodChannel.invokeMethod(
        'delConversation', <String, String>{'identifier': identifier});
  }

  ///获取一个会话的消息，暂不支持流式查询 Get a message for a session, no streaming query is supported at this time
  ///identifier 会话id
  ///count 获取消息数量 ,默认50条
  ///ctype 1 私信，2群聊  ,默认是私信
  Future<dynamic> getMessages(String identifier,
      [int count = 50, int ctype = 1]) async {
    return await _methodChannel.invokeMethod('getMessages', <String, dynamic>{
      'identifier': identifier,
      'count': count,
      'ctype': ctype
    });
  }

  ///发送文本消息
  ///Send a text message
  Future<dynamic> sendTextMessages(String identifier, String content,
      {int type}) async {
    return await _methodChannel
        .invokeMethod('sendTextMessages', <String, dynamic>{
      'identifier': identifier,
      'content': content,
      'type': type ?? 1,
    });
  }

  ///发送图片消息
  ///imagePath   eg for android : Environment.getExternalStorageDirectory() + "/DCIM/Camera/1.jpg"
  Future<dynamic> sendImageMessages(String identifier, String imagePath,
      {int type}) async {
    return await _methodChannel
        .invokeMethod('sendImageMessages', <String, dynamic>{
      'identifier': identifier,
      'image_path': imagePath,
      'type': type ?? 1,
    });
  }

  ///发送语音消息
  ///soundPath   eg for android : Environment.getExternalStorageDirectory() + "/sound.mp3"
  Future<dynamic> sendSoundMessages(
      String identifier, String soundPath, int type,
      [int duration = 10]) async {
    return await _methodChannel
        .invokeMethod('sendSoundMessages', <String, dynamic>{
      'identifier': identifier,
      'sound_path': soundPath,
      'duration': duration,
      'type': type ?? 1,
    });
  }

  ///发送视频消息 （封面和视频必须是在本地存在的，不能直接放网络远程的文件，否则7006）duration为毫秒1秒等于1000毫秒
  ///Send video messages (cover and video must be local, can not directly put the network remote file, otherwise 7006)
  ///duration is milliseconds 1 second equals 1000 milliseconds
  Future<dynamic> buildVideoMessage(String identifier, String imgPath,
      String videoPath, int width, int height, int type,
      [int duration = 10]) async {
    return await _methodChannel
        .invokeMethod('buildVideoMessage', <String, dynamic>{
      'imgPath': imgPath,
      'identifier': identifier,
      'videoPath': videoPath,
      'width': width,
      'height': height,
      'duration': duration,
      'type': type ?? 1,
    });
  }

  ///发送位置消息
  ///eg：
  ///lat 113.93
  ///lng 22.54
  ///desc 腾讯大厦
  Future<dynamic> sendLocationMessages(
      String identifier, double lat, double lng, String desc,
      {int type}) async {
    return await _methodChannel.invokeMethod('sendLocation', <String, dynamic>{
      'identifier': identifier,
      'lat': lat,
      'lng': lng,
      'desc': desc,
      'type': type ?? 1,
    });
  }

  ///添加好友
  ///add friend
  Future<dynamic> addFriend(String identifier) async {
    return await _methodChannel
        .invokeMethod("addFriend", <String, dynamic>{'identifier': identifier});
  }

  ///删除好友
  ///delete friend
  Future<dynamic> delFriend(String identifier) async {
    return await _methodChannel
        .invokeMethod("delFriend", <String, dynamic>{'identifier': identifier});
  }

  ///获取好友列表
  ///Get a list of friends
  Future<dynamic> listFriends(String identifier) async {
    return await _methodChannel.invokeMethod(
        "listFriends", <String, dynamic>{'identifier': identifier});
  }

  ///处理好友的请求，接受/拒绝
  ///opTypeStr 接受传 Y
  ///opTypeStr 拒绝传 N
  Future<dynamic> opFriend(String identifier, String opTypeStr) async {
    return await _methodChannel.invokeMethod("opFriend",
        <String, dynamic>{'identifier': identifier, 'opTypeStr': opTypeStr});
  }

  ///获取用户资料
  ///param user is a list ["usersf1","jiofoea2"]
  Future<dynamic> getUsersProfile(List<String> users) async {
    return await _methodChannel
        .invokeMethod("getUsersProfile", <String, dynamic>{'users': users});
  }

  ///设置个人资料
  ///Set up profile
  Future<dynamic> setUsersProfile(
      int gender, String nick, String faceUrl) async {
    return await _methodChannel.invokeMethod("setUsersProfile",
        <String, dynamic>{'gender': gender, 'nick': nick, 'faceUrl': faceUrl});
  }

  ///发起群聊
  ///createGroupChat commit up
  Future<dynamic> createGroupChat(
      {String groupID,
      String name,
      String faceUrl,
      String introduction,
      String notification,
      String member1,
      String member2,
      String member3,
      String member4}) async {
    return await _methodChannel
        .invokeMethod("createGroupChat", <String, dynamic>{
      'groupID': groupID,
      'member1': member1,
      'member2': member2 ?? '',
      'member3': member3 ?? '',
      'member4': member4 ?? '',
      'name': name ?? member1,
      'faceUrl': faceUrl ??
          'https://c-ssl.duitang.com/uploads/item/201803/18/20180318113721_jtrjs.jpg',
      'introduction': introduction ?? '这是群简介',
      'notification': notification ?? '这是群公告',
    });
  }

  ///修改好友备注
  ///Edit friend notes
  Future<dynamic> editFriendNotes(String identifier, String remarks) async {
    return await _methodChannel.invokeMethod('editFriendNotes',
        <String, dynamic>{'identifier': identifier, 'remarks': remarks});
  }

  ///获取好友备注
  ///getRemark
  Future<dynamic> getRemark(String identifier) async {
    return await _methodChannel
        .invokeMethod('getRemark', <String, dynamic>{'identifier': identifier});
  }

  ///删除好友
  ///deleteFriend
  Future<dynamic> deleteFriend(String id) async {
    return await _methodChannel
        .invokeMethod('deleteFriend', <String, dynamic>{'id': id});
  }

  ///把好友移动至黑名单   多个可以用,分隔
  ///Move friends to blacklist multiples, separate
  Future<dynamic> addBlack(String mId) async {
    return await _methodChannel
        .invokeMethod('addBlack', <String, dynamic>{'mId': mId});
  }

  ///把好友移除黑名单  多个可以用,分隔
  ///deleteBlackList
  Future<dynamic> deleteBlackList(String mId) async {
    return await _methodChannel
        .invokeMethod('deleteBlackList', <String, dynamic>{'mId': mId});
  }

  /// 获取黑名单列表
  /// getBlackList
  Future<dynamic>getBlackList()async{
    return await _methodChannel
        .invokeMethod('getBlackList');
  }

  ///测试使用eventChannel推送数据过来
  ///Test using eventChannel to push data over
  Future<dynamic> postDataTest() async {
    return await _methodChannel.invokeMethod("post_data_test");
  }

  dynamic _parseBatteryState(event) {
    return event;
  }
}
